package solutions.bitbadger.documents.scala

import org.scalatest.funspec.AnyFunSpec
import org.scalatest.matchers.should.Matchers
import solutions.bitbadger.documents.scala.support.{ByteIdClass, IntIdClass, LongIdClass, ShortIdClass, StringIdClass}
import solutions.bitbadger.documents.{AutoId, DocumentException}

class AutoIdSpec extends AnyFunSpec with Matchers {

  describe("generateUUID") {
    it("generates a UUID string") {
      AutoId.generateUUID().length shouldEqual 32
    }
  }

  describe("generateRandomString") {
    it("generates a random hex character string of an even length") {
      AutoId.generateRandomString(8).length shouldEqual 8
    }
    it("generates a random hex character string of an odd length") {
      AutoId.generateRandomString(11).length shouldEqual 11
    }
    it("generates different random hex character strings") {
      val result1 = AutoId.generateRandomString(16)
      val result2 = AutoId.generateRandomString(16)
      result1 should not be theSameInstanceAs (result2)
    }
  }

  describe("needsAutoId") {
    it("fails for null document") {
      an [DocumentException] should be thrownBy AutoId.needsAutoId(AutoId.DISABLED, null, "id")
    }
    it("fails for missing ID property") {
      an [DocumentException] should be thrownBy AutoId.needsAutoId(AutoId.UUID, IntIdClass(0), "Id")
    }
    it("returns false if disabled") {
      AutoId.needsAutoId(AutoId.DISABLED, "", "") shouldBe false
    }
    it("returns true for Number strategy and byte ID of 0") {
      AutoId.needsAutoId(AutoId.NUMBER, ByteIdClass(0), "id") shouldBe true
    }
    it("returns false for Number strategy and byte ID of non-0") {
      AutoId.needsAutoId(AutoId.NUMBER, ByteIdClass(77), "id") shouldBe false
    }
    it("returns true for Number strategy and short ID of 0") {
      AutoId.needsAutoId(AutoId.NUMBER, ShortIdClass(0), "id") shouldBe true
    }
    it("returns false for Number strategy and short ID of non-0") {
      AutoId.needsAutoId(AutoId.NUMBER, ShortIdClass(31), "id") shouldBe false
    }
    it("returns true for Number strategy and int ID of 0") {
      AutoId.needsAutoId(AutoId.NUMBER, IntIdClass(0), "id") shouldBe true
    }
    it("returns false for Number strategy and int ID of non-0") {
      AutoId.needsAutoId(AutoId.NUMBER, IntIdClass(6), "id") shouldBe false
    }
    it("returns true for Number strategy and long ID of 0") {
      AutoId.needsAutoId(AutoId.NUMBER, LongIdClass(0), "id") shouldBe true
    }
    it("returns false for Number strategy and long ID of non-0") {
      AutoId.needsAutoId(AutoId.NUMBER, LongIdClass(2), "id") shouldBe false
    }
    it("fails for Number strategy and non-number ID") {
      an [DocumentException] should be thrownBy AutoId.needsAutoId(AutoId.NUMBER, StringIdClass(""), "id")
    }
    it("returns true for UUID strategy and blank ID") {
      AutoId.needsAutoId(AutoId.UUID, StringIdClass(""), "id") shouldBe true
    }
    it("returns false for UUID strategy and non-blank ID") {
      AutoId.needsAutoId(AutoId.UUID, StringIdClass("howdy"), "id") shouldBe false
    }
    it("fails for UUID strategy and non-string ID") {
      an [DocumentException] should be thrownBy AutoId.needsAutoId(AutoId.UUID, IntIdClass(5), "id")
    }
    it("returns true for Random String strategy and blank ID") {
      AutoId.needsAutoId(AutoId.RANDOM_STRING, StringIdClass(""), "id") shouldBe true
    }
    it("returns false for Random String strategy and non-blank ID") {
      AutoId.needsAutoId(AutoId.RANDOM_STRING, StringIdClass("full"), "id") shouldBe false
    }
    it("fails for Random String strategy and non-string ID") {
      an [DocumentException] should be thrownBy AutoId.needsAutoId(AutoId.RANDOM_STRING, ShortIdClass(55), "id")
    }
  }
}
